#include <avr/io.h>
#include <ctype.h>
#include <stdlib.h>
#include <avr/pgmspace.h>

#define MAX_LEDS 5

#define MAX_PWM_COUNT   32
#define MAX_PWM_VALUE   256

uint8_t pwm_table[MAX_PWM_COUNT] PROGMEM =
  {  0,   1,   2,   2,   2,   3,   3,   4,
     5,   6,   7,   8,  10,  11,  13,  16,
    19,  23,  27,  32,  38,  45,  54,  64,
    76,  91, 108, 128, 152, 181, 215, 255
  };

unsigned int tick = 0;

void led_off(unsigned char nr)
{
  PORTB |= (1 << nr);
}


int main(void)
{
  unsigned char delay_counter = 0;
  signed char index[MAX_LEDS];
  unsigned char pwm_value[MAX_LEDS];
  unsigned char i;

  // pin 0..4 as output
  DDRB |= 0x1f;

  for (i=0; i < MAX_LEDS; ++i)
  {
    index[i] = rand()%MAX_PWM_VALUE;
    pwm_value[i] = pgm_read_byte(&pwm_table[index[i]]);
  }

  while (1)
  {
    if (tick == 0)
    {
      PORTB &= 0xff << MAX_LEDS;  // leds ein
    }

    for ( i = 0; i < MAX_LEDS; ++i)
    {
      if ( tick == pwm_value[i] )
      {
        led_off(i);  // leds aus
      }
    }	  

    // pwm period reached --> calculate next pwm cycle
    if (++tick == MAX_PWM_VALUE)
    {
      tick = 0;
      if (delay_counter++ >= 10)
      {
        delay_counter = 0;
        for (i=0; i < MAX_LEDS; ++i)
        {
          index[i] += -3 + rand() %8;
          if (index[i] < 0) index[i] += (rand() %8);
          if (index[i] > MAX_PWM_COUNT) index[i] -= (rand() %8);
          pwm_value[i] = pgm_read_byte(&pwm_table[index[i]]);
        }
      }
    }

  } // while(1)
} // main
